#!/usr/bin/env node
var isOld = _isOldVersion();

var branches;
var buffer;
var Events = require('events');

var ee = isOld ? Events.EventEmitter : new Events();
var emit = ee.emit;
var exec = require('child_process').exec;
var on = ee.on;
var prompt = require('prompt');
var Q = require('q');
var regEx;

_initializeEvents();

if(process.argv.length < 3) {
  console.error('Must provide an expression.');
  process.exit(1);
}

regEx = process.argv[2];

if(regEx === '-a' || regEx === '--all') {
  regEx = '-v ^.*master$'
}

emit('get-buffer', regEx);

function _askToProceed() {
  var defer = Q.defer();

  branches.map(function mapBranches(branch) {
    console.log('- ' + branch);
  });

  console.log('==============================');
  console.log('Would you like to delete the selected branches?');

  prompt.start();

  prompt.get({
    properties: [{
        name: 'yesOrNo',
        description: '(y/n)',
        pattern: /^(y|n)$/,
        message: 'Only y or n are valid answers',
        required: true
      }]
  }, function proceedCallback(err, results) {
    if(results[0] === 'y') {
      defer.resolve();
    } else {
      defer.reject();
      console.warn('Exiting...');
      process.exit(0);
    }
  });

  return defer.promise;
}

function _getBuffer(regEx) {
  var buffer;

  try {
    exec('git branch | grep ' + regEx + ' | tr "\n" ","', function bufferCallback(err, buffer, stderr) {
      emit('parse-buffer', buffer);
    });
  } catch(e) {
    console.error('The regular expression provided returns 0 matches.');
    process.exit(1);
  }

  return buffer;
}

function _isError(err) {
  return (err && err.code === 1);
}

function _isOldVersion() {
  var v = /^v(?:[0-9])\.([0-9]{2})\./.exec(process.version)[1];

  return v < 12;
}

function _parseBuffer(buffer) {
  try {
    var output = buffer.toString();

    output = output.substr(0, output.lastIndexOf(','))
    .replace(/[*] /, '')
    .replace(/\s\s/g, '');

    return output.split(',');
  } catch(e) {
    emit('parse-buffer:error', e);
  }
}

function _removeBranches(branches) {
  branches.map(function(branch) {
    _removeLocalBranch(branch);
    _removeRemoteBranch(branch);
  });

  function _removeLocalBranch(branch) {
    exec('git branch -D ' + branch, function removeRemoveCallback(err, out) {
      if(_isError(err)) {
        emit('local-branch-removed:error', err, branch);
      } else {
        emit('local-branch-removed:success', branch);
      }
    });
  }

  function _removeRemoteBranch(branch) {
    exec('git push origin :' + branch, function removeRemoveCallback(err, out) {
      if(_isError(err)) {
        emit('remote-branch-removed:error', err, branch);
      } else {
        emit('remote-branch-removed:success', branch);
      }
    });
  }
}

function _switchToMaster(cb) {
  exec('git co master', function switchToMaster(err, out) {
    if(_isError(err)) {
      emit('switch-to-master:error', err);
    } else {
      emit('switch-to-master:success');
    }
  });
}

function _initializeEvents() {
  on('get-buffer', function getBufferEvent(regEx) {
    _getBuffer(regEx);
  });

  on('parse-buffer', function parseBufferEvent(buffer) {
    branches = _parseBuffer(buffer);
    emit('ask-to-proceed');
  });

  on('parse-buffer:error', function parseBufferError() {
    console.log('Unable to parse results... exiting.');
    process.exit(1);
  });

  on('ask-to-proceed', function askToProceedEvent() {
    _askToProceed().then(function askToProceedCallback(response) {
      emit('switch-to-master');
    });
  });

  on('switch-to-master', function switchToMasterEvent() {
    _switchToMaster();
  });

  on('switch-to-master:error', function switchToMasterError(e) {
    console.log('Failed to switch to master branch... exiting.'+ '\n' + e.message);

    process.exit(1);
  });

  on('switch-to-master:success', function switchToMasterSuccess() {
    emit('remove-branches', branches);
  });

  on('remove-branches', function removeBranchesEvent(branches) {
    _removeBranches(branches);
  });

  on('local-branch-removed:success', function removeLocalSuccessEvent(branch) {
    console.log('Local branch removed: ' + branch);
  });

  on('local-branch-removed:error', function removeLocalErrorEvent(e, branch) {
    console.log('Failed to remove local branch: ' + branch+ '\n' + e.message);
  });

  on('remote-branch-removed:success', function removeRemoteSuccessEvent(branch) {
    console.log('Remote branch removed: ' + branch);
  });

  on('remote-branch-removed:error', function removeRemoteSuccessError(e, branch) {
    console.log('Failed to remove remote branch: ' + branch + '\n' + e.message);
  });
}
